#include <GUIConstantsEx.au3>
#include <WinAPI.au3>
#include <WindowsConstants.au3>

Global $Resize_Windows[1][11]
Global Const $RESIZE_HWND		= 0
Global Const $RESIZE_PARENTHWND	= 1
Global Const $RESIZE_DOCK		= 2
Global Const $RESIZE_POSX1		= 3
Global Const $RESIZE_POSX2		= 4
Global Const $RESIZE_POSY1		= 5
Global Const $RESIZE_POSY2		= 6
Global Const $RESIZE_WIDTH		= 7
Global Const $RESIZE_HEIGHT		= 8
Global Const $RESIZE_WWIDTH		= 9
Global Const $RESIZE_WHEIGHT	= 10

; #FUNCTION# ====================================================================================================
; Name...........:	_Resize_AddWindow
; Description....:	quivalent GUICtrlSetResizing pour une fentre $WM_CHILD
; Syntax.........:	_Resize_AddWindow($hWnd, $Dock)
;
; Parameters.....:	$hWnd		- Handle de la fentre enfant
; 					$Dock		- Toutes les veleurs $GUI_DOCKXXXX
;
; Return values..:	Aucune
;
; Author.........:	TommyDDR - http://tom-barthelemy.fr
; Modified.......:
; Remarks........:
;
; Related........:
; Link...........:
; Example........:
; ===============================================================================================================
Func _Resize_AddWindow($hWnd, $Dock)
	Local $hParent = _WinAPI_GetParent($hWnd)
	If($hParent = 0) Then
		Return SetError(1, 0, False)
	EndIf
	Local $Indice = UBound($Resize_Windows, 1)-1
	If($Indice = 0) Then
		_GUIRegisterMsg($WM_SIZE, "__Func_WM_SIZE")
	EndIf
	Local $ClientRelativePos = GetClientRelativePos($hWnd)
	Local $ParentPos = GetClientPos($hParent)

	$Resize_Windows[$Indice][$RESIZE_HWND]			= $hWnd
	$Resize_Windows[$Indice][$RESIZE_PARENTHWND]	= $hParent
	$Resize_Windows[$Indice][$RESIZE_DOCK]			= $Dock
	$Resize_Windows[$Indice][$RESIZE_WWIDTH]		= $ParentPos[2]
	$Resize_Windows[$Indice][$RESIZE_WHEIGHT]		= $ParentPos[3]
	$Resize_Windows[$Indice][$RESIZE_WIDTH]			= $ClientRelativePos[2]
	$Resize_Windows[$Indice][$RESIZE_HEIGHT]		= $ClientRelativePos[3]
	If(BitAND($Dock, $GUI_DOCKLEFT)) Then
		$Resize_Windows[$Indice][$RESIZE_POSX1]	= $ClientRelativePos[0]
	Else
		$Resize_Windows[$Indice][$RESIZE_POSX1]	= $ClientRelativePos[0]/$Resize_Windows[$Indice][$RESIZE_WWIDTH]
	EndIf
	If(BitAND($Dock, $GUI_DOCKRIGHT)) Then
		$Resize_Windows[$Indice][$RESIZE_POSX2]	= $Resize_Windows[$Indice][$RESIZE_WWIDTH]-($ClientRelativePos[0]+$Resize_Windows[$Indice][$RESIZE_WIDTH])
	Else
		$Resize_Windows[$Indice][$RESIZE_POSX2]	= ($Resize_Windows[$Indice][$RESIZE_WWIDTH]-($ClientRelativePos[0]+$Resize_Windows[$Indice][$RESIZE_WIDTH]))/$Resize_Windows[$Indice][$RESIZE_WWIDTH]
	EndIf
	If(BitAND($Dock, $GUI_DOCKTOP)) Then
		$Resize_Windows[$Indice][$RESIZE_POSY1]	= $ClientRelativePos[1]
	Else
		$Resize_Windows[$Indice][$RESIZE_POSY1]	= $ClientRelativePos[1]/$Resize_Windows[$Indice][$RESIZE_WHEIGHT]
	EndIf
	If(BitAND($Dock, $GUI_DOCKBOTTOM)) Then
		$Resize_Windows[$Indice][$RESIZE_POSY2]	= $Resize_Windows[$Indice][$RESIZE_WHEIGHT]-($ClientRelativePos[1]+$Resize_Windows[$Indice][$RESIZE_HEIGHT])
	Else
		$Resize_Windows[$Indice][$RESIZE_POSY2]	= ($Resize_Windows[$Indice][$RESIZE_WHEIGHT]-($ClientRelativePos[1]+$Resize_Windows[$Indice][$RESIZE_HEIGHT]))/$Resize_Windows[$Indice][$RESIZE_WHEIGHT]
	EndIf
	If(Not(BitAND($Dock, $GUI_DOCKWIDTH) And Not(BitAND($Dock, $GUI_DOCKLEFT) And BitAND($Dock, $GUI_DOCKRIGHT)))) Then
		$Resize_Windows[$Indice][$RESIZE_WWIDTH] = 0
	EndIf
	If(Not(BitAND($Dock, $GUI_DOCKHEIGHT) And Not(BitAND($Dock, $GUI_DOCKHEIGHT) And BitAND($Dock, $GUI_DOCKHEIGHT)))) Then
		$Resize_Windows[$Indice][$RESIZE_WWIDTH] = 0
	EndIf
	ReDim $Resize_Windows[UBound($Resize_Windows, 1)+1][UBound($Resize_Windows, 2)]
EndFunc

; #FUNCTION# ====================================================================================================
; Name...........:	_Resize_StopWindow
; Description....:	Stop un redimensionnement
; Syntax.........:	_Resize_StopWindow($hWnd)
;
; Parameters.....:	$hWnd		- Handle de la fentre enfant
;
; Return values..:	Aucune
;
; Author.........:	TommyDDR - http://tom-barthelemy.fr
; Modified.......:
; Remarks........:
;
; Related........:
; Link...........:
; Example........:
; ===============================================================================================================
Func _Resize_StopWindow($hWnd)
	Local $Cible = _Resize_VerifHwnd($hWnd)
	If($Cible >= 0) Then
		For $j = $Cible To UBound($Resize_Windows, 1)-2
			For $i = $Cible To UBound($Resize_Windows, 2)-2
				$Resize_Windows[$j][$i] = $Resize_Windows[$j+1][$i]
			Next
		Next
		ReDim $Resize_Windows[UBound($Resize_Windows, 1)-1][UBound($Resize_Windows, 2)]
		If(UBound($Resize_Windows, 1) = 1) Then
			_GUIUnRegisterMsg($WM_SIZE, "__Func_WM_SIZE")
		EndIf
	EndIf
EndFunc

Func __Func_WM_SIZE($hWnd, $iMsg, $iwParam, $ilParam)
	Local $loWord = _WinAPI_LoWord($ilParam)
	Local $hiWord = _WinAPI_HiWord($ilParam)
	If($loWord <> 0 And $hiWord <> 0) Then
		For $i = 0 To UBound($Resize_Windows, 1)-2
			If($hWnd = $Resize_Windows[$i][$RESIZE_PARENTHWND]) Then
				Local $ClientRelativePos = GetClientRelativePos($Resize_Windows[$i][$RESIZE_HWND])
				Local $ParentPos = GetClientPos($Resize_Windows[$i][$RESIZE_PARENTHWND])
				Local $NewPos[4]
				Local $Dock = $Resize_Windows[$i][$RESIZE_DOCK]
				If(BitAND($Dock, $GUI_DOCKLEFT)) Then
					$NewPos[0] = $Resize_Windows[$i][$RESIZE_POSX1]
				Else
					$NewPos[0] = $ParentPos[2]*$Resize_Windows[$i][$RESIZE_POSX1]
				EndIf
				If(BitAND($Dock, $GUI_DOCKRIGHT)) Then
					$NewPos[2] = $ParentPos[2]-$Resize_Windows[$i][$RESIZE_POSX2]
				Else
					$NewPos[2] = ($ParentPos[2]-($ParentPos[2]*$Resize_Windows[$i][$RESIZE_POSX2]))
				EndIf
				If(BitAND($Dock, $GUI_DOCKTOP)) Then
					$NewPos[1] = $Resize_Windows[$i][$RESIZE_POSY1]
				Else
					$NewPos[1] = $ParentPos[3]*$Resize_Windows[$i][$RESIZE_POSY1]
				EndIf
				If(BitAND($Dock, $GUI_DOCKBOTTOM)) Then
					$NewPos[3] = $ParentPos[3]-$Resize_Windows[$i][$RESIZE_POSY2]
				Else
					$NewPos[3] = ($ParentPos[3]-($ParentPos[3]*$Resize_Windows[$i][$RESIZE_POSY2]))
				EndIf
				If(BitAND($Dock, $GUI_DOCKWIDTH)) Then
					If(BitAND($Dock, $GUI_DOCKRIGHT)) Then
						$NewPos[0] = $NewPos[2]-$Resize_Windows[$i][$RESIZE_WIDTH]
					Else
						$NewPos[2] = $NewPos[0]+$Resize_Windows[$i][$RESIZE_WIDTH]
					EndIf
				EndIf
				If(BitAND($Dock, $GUI_DOCKHEIGHT)) Then
					If(BitAND($Dock, $GUI_DOCKBOTTOM)) Then
						$NewPos[1] = $NewPos[3]-$Resize_Windows[$i][$RESIZE_HEIGHT]
					Else
						$NewPos[3] = $NewPos[1]+$Resize_Windows[$i][$RESIZE_HEIGHT]
					EndIf
				EndIf
				;WinMove($Resize_Windows[$i][$RESIZE_HWND], "", $NewPos[0], $NewPos[1], $NewPos[2]-$NewPos[0], $NewPos[3]-$NewPos[1])
				_RESIZER( $Resize_Windows[$i][$RESIZE_HWND], $NewPos[0], $NewPos[1], $NewPos[2]-$NewPos[0], $NewPos[3]-$NewPos[1])

			EndIf
		Next
	EndIf
	Return $GUI_RUNDEFMSG
EndFunc

Func _Resize_VerifHwnd($hWnd)
	Local $Cible = -1
	For $i = 0 To UBound($Resize_Windows, 1)-2
		If($Resize_Windows[$i][$RESIZE_HWND] = $hWnd) Then
			$Cible = $i
			ExitLoop
		EndIf
	Next
	Return $Cible
EndFunc

; ==== _GUIRegisterMsg.au3 ================================================================================

#include-once

Dim $UDF_GUIRegisterMsg[1][2]
Dim $UDF_ListeRegisters[1]

Func _GUIRegisterMsg($Msg, $Fonction)
    ;Si la fonctoin n'a pas encore t register pour le message donn
    If(__GUIRegisterGetRegister($Msg, $Fonction) = -1) Then
        ;On ajoute la fonction et le message dans la liste
        __GUIRegisterAddMsg($Msg)
        Local $NumeroRegister = UBound($UDF_GUIRegisterMsg, 1)-1
        $UDF_GUIRegisterMsg[$NumeroRegister][0] = $Msg
        $UDF_GUIRegisterMsg[$NumeroRegister][1] = $Fonction
        ReDim $UDF_GUIRegisterMsg[UBound($UDF_GUIRegisterMsg, 1)+1][UBound($UDF_GUIRegisterMsg, 2)]
    EndIf
EndFunc

Func _GUIUnRegisterMsg($Msg, $Fonction)
    Local $Numero = __GUIRegisterGetRegister($Msg, $Fonction)
    ; Si la fonction associe au message existe
    If($Numero >= 0) Then
        ; On la supprime de la liste
        __GUIRegisterDelRegister($Numero)
    EndIf
EndFunc

Func __GUIRegisterAddMsg($Msg)
    Local $IDMsg = -1
    ; On cherche si le message  dj t register
    For $i = 0 To UBound($UDF_ListeRegisters, 1)-2
        If($UDF_ListeRegisters[$i] = $Msg) Then
            $IDMsg = $i
            ExitLoop
        EndIf
    Next
    ; Si le message n'a pas encore t register
    If($IDMsg = -1) Then
        $UDF_ListeRegisters[UBound($UDF_ListeRegisters, 1)-1] = $Msg
        GUIRegisterMsg($Msg, "__Func_GUIRegisterMsg")
    EndIf
EndFunc

Func __Func_GUIRegisterMsg($hWnd, $Msg, $wParam, $lParam)
;~  ConsoleWrite("Hehe : " & $hWnd & ", " & $Msg & ", " & $wParam & ", " & $lParam & @CRLF)
    Local $Return = $GUI_RUNDEFMSG
    Local $ReturnBis
    For $i = 0 To UBound($UDF_GUIRegisterMsg, 1)-2
        If($UDF_GUIRegisterMsg[$i][0] = $Msg) Then
            $ReturnBis = Call($UDF_GUIRegisterMsg[$i][1], $hWnd, $Msg, $wParam, $lParam)
            If($ReturnBis <> $GUI_RUNDEFMSG) Then
                $Return = $ReturnBis
            EndIf
        EndIf
    Next
    Return $Return
EndFunc

Func __GUIRegisterGetRegister($Msg, $Fonction)
    Local $Retour = -1
    For $i = 0 To UBound($UDF_GUIRegisterMsg, 1)-2
        If($UDF_GUIRegisterMsg[$i][0] = $Msg And $UDF_GUIRegisterMsg[$i][1] = $Fonction) Then
            $Retour = $i
            ExitLoop
        EndIf
    Next
    Return $Retour
EndFunc

Func __GUIRegisterDelRegister($Numero)
    For $i = $Numero To UBound($UDF_GUIRegisterMsg, 1)-3
        $UDF_GUIRegisterMsg[$i][0] = $UDF_GUIRegisterMsg[$i+1][0]
        $UDF_GUIRegisterMsg[$i][1] = $UDF_GUIRegisterMsg[$i+1][1]
    Next
    ReDim $UDF_GUIRegisterMsg[UBound($UDF_GUIRegisterMsg, 1)-1][UBound($UDF_GUIRegisterMsg, 2)]
EndFunc

; ==== WindowsEx.au3 ====================================================================================

Func GetClientRelativePos($hWnd)
	Local $hParent = _WinAPI_GetParent($hWnd)
	If($hParent = 0) Then
		Return SetError(1, 0, False)
	EndIf
	Local $parentPos = GetClientPos($hParent)
	Local $clientPos = GetClientPos($hWnd)
	Local $clientRelativePos[4] = [$clientPos[0]-$parentPos[0], $clientPos[1]-$parentPos[1], $clientPos[2], $clientPos[3]]
	Return $clientRelativePos
EndFunc

Func GetClientPos($hWnd)
	Local $Side = GetSideSize($hWnd)
	Local $wPos = WinGetPos($hWnd)
	Local $clientPos[4] = [$wPos[0]+$Side[0], $wPos[1]+$Side[1], $wPos[2]-$Side[0]*2, $wPos[3]-$Side[0]-$Side[1]]
	Return $clientPos
EndFunc

Func GetSideSize($hWnd, $dim = -1)
	Local $clientSize = WinGetClientSize($hWnd)
	Local $winSize = WinGetPos($hWnd)
	Local $size[2]
	$size[0] = Round(($winSize[2]-$clientSize[0])/2)
	$size[1] = $winSize[3]-$clientSize[1]-$size[0]
	Switch $dim
		Case 0
			$size = Round(($winSize[2]-$clientSize[0])/2)
		Case 1
			$size = $winSize[3]-$clientSize[1]-$size[0]
	EndSwitch
	Return $size
EndFunc

Func GetNormalWindowsSide($dim = -1)
	Local $Gui = WinGetHandle(AutoItWinGetTitle())
	Local $SideSize = GetSideSize($Gui, $dim)
	Return $SideSize
EndFunc